#include "testlib.h"
#include <iostream>
#include <vector>
#include <set>

using namespace std;

vector<long long> get_queries_for_pow2(long long pow2, int max_from_group) {
  if (pow2 == 1) {
    return vector<long long>(1, 1);
  }
  long long half = pow2 >> 1;
  vector<long long> queries = get_queries_for_pow2(half, max_from_group);
  for (int i = 0; i < queries.size(); ++i) {
    queries[i] = 2 * queries[i];
  }

  if (half <= max_from_group) {
    for (int i = 0; i < half; ++i) {
      queries.push_back(i * 2 + 1);
    }
  } else {    
    set<long long> query_set;
    for (int i = 0; i < max_from_group; ++i) {
      query_set.insert(rnd.next(1LL, half) * 2 - 1);
      query_set.insert(i * 2 + 1);
      query_set.insert((half - i) * 2 - 1);
    }
    queries.insert(queries.end(), query_set.begin(), query_set.end());
  }
  
  return queries;
}

int main(int argc, char* argv[]) {
  registerGen(argc, argv, 1);

  if (argc != 5) {
    cerr << "Usage: n, q, max_from_group, seed" << endl;
    return 1;
  }

  long long n = atoll(argv[1]);
  int q = atoi(argv[2]);
  int max_from_group = atoi(argv[3]);
  cout << n << ' ' << q << endl;

  long long lower_pow2 = 1, residue;
  while ((lower_pow2 << 1) <= n) {
    lower_pow2 <<= 1;
  }
  residue = n - lower_pow2;

  // checking all jumps
  vector<long long> queries = get_queries_for_pow2(lower_pow2, max_from_group);
  for (int i = 0; i < queries.size(); ++i) {
    long long query = queries[i] + 2 * residue;
    if (query > n) {
      queries[i] = 2 * (query - n);
    }
  }

  set<long long> query_set(queries.begin(), queries.end());

  // more from head
  if (residue <= max_from_group) {
    for (int i = 0; i < residue; ++i) {
      query_set.insert(2 * i + 1);
      query_set.insert(2 * i + 2);
    }
  } else {
    for (int i = 0; i < max_from_group; ++i) {
      query_set.insert(2 * rnd.next(1LL, residue));
      query_set.insert(2 * rnd.next(1LL, residue) - 1);
    }
  }

  // border regions
  for (int i = 0; i < max_from_group; ++i) {
    query_set.insert(min((long long)i + 1, n));
    query_set.insert(max(n - i, 1LL));
    query_set.insert(max(2 * residue - i, 1LL));
  }

  // adding random
  while (query_set.size() < q) {
    query_set.insert(rnd.next(1LL, n));
  }

  vector<long long> final_queries(query_set.begin(), query_set.end());
  shuffle(final_queries.begin(), final_queries.end());
  for (int i = 0; i < final_queries.size(); ++i) {
    cout << final_queries[i] << endl;
  }
  return 0;
}
